#!/usr/bin/env python
# -*- coding: utf-8  -*-
################################################################################
#
#  Rattail -- Retail Software Framework
#  Copyright © 2010-2012 Lance Edgar
#
#  This file is part of Rattail.
#
#  Rattail is free software: you can redistribute it and/or modify it under the
#  terms of the GNU Affero General Public License as published by the Free
#  Software Foundation, either version 3 of the License, or (at your option)
#  any later version.
#
#  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for
#  more details.
#
#  You should have received a copy of the GNU Affero General Public License
#  along with Rattail.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################

"""
``rattail.pyramid.views.departments`` -- Department Views
"""

from pyramid.view import view_config

from edbob.pyramid import filters
from edbob.pyramid import forms
from edbob.pyramid import grids
from edbob.pyramid import Session

import rattail


@view_config(route_name='department.list', renderer='/departments/index.mako')
def list_departments(context, request):

    fmap = filters.get_filter_map(
        rattail.Department,
        exact=['number'],
        ilike=['name'])

    config = filters.get_search_config(
        'department.list', request, fmap,
        include_filter_name=True,
        filter_type_name='lk')

    search = filters.get_search_form(config)

    config = grids.get_grid_config(
        'department.list', request, search,
        filter_map=fmap, sort='name')

    smap = grids.get_sort_map(
        rattail.Department,
        ['number', 'name'])

    def query(config):
        q = Session.query(rattail.Department)
        q = filters.filter_query(q, config)
        q = grids.sort_query(q, config, smap)
        return q

    departments = grids.get_pager(query, config)
    g = forms.AlchemyGrid(
        rattail.Department, departments, config,
        gridurl=request.route_url('department.list'))

    g.configure(
        include=[
            g.number,
            g.name,
            ],
        readonly=True)

    grid = g.render(class_='clickable departments')
    return grids.render_grid(request, grid, search)


def includeme(config):
    config.add_route('department.list', '/departments')
    config.scan(__name__)
