#!/usr/bin/env python
# -*- coding: utf-8  -*-
################################################################################
#
#  Rattail -- Retail Software Framework
#  Copyright © 2010-2012 Lance Edgar
#
#  This file is part of Rattail.
#
#  Rattail is free software: you can redistribute it and/or modify it under the
#  terms of the GNU Affero General Public License as published by the Free
#  Software Foundation, either version 3 of the License, or (at your option)
#  any later version.
#
#  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for
#  more details.
#
#  You should have received a copy of the GNU Affero General Public License
#  along with Rattail.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################

"""
``rattail.pyramid.views.products`` -- Product Views
"""

from sqlalchemy.orm import joinedload

from pyramid.view import view_config

from edbob.pyramid import filters
from edbob.pyramid import forms
from edbob.pyramid import grids
from edbob.pyramid import Session

import rattail
from rattail.pyramid.forms import UpcFieldRenderer


@view_config(route_name='products', renderer='/products/index.mako')
def products(context, request):

    fmap = filters.get_filter_map(
        rattail.Product,
        exact=['upc'],
        ilike=['description'],
        brand=filters.filter_ilike(rattail.Brand.name))

    config = filters.get_search_config(
        'products', request, fmap,
        include_filter_brand=True,
        filter_type_brand='lk',
        include_filter_description=True,
        filter_type_description='lk')

    search = filters.get_search_form(
        config, upc="UPC")

    config = grids.get_grid_config(
        'products', request, search,
        filter_map=fmap, sort='description')

    smap = grids.get_sort_map(
        rattail.Product,
        ['upc', 'description'],
        brand=grids.sorter(rattail.Brand.name))

    jmap = {
        'brand':
            lambda q: q.join(rattail.Brand),
        }

    def query(config):
        q = Session.query(rattail.Product)
        q = q.options(joinedload(rattail.Product.brand))
        q = filters.filter_query(q, config, jmap)
        q = grids.sort_query(q, config, smap, jmap)
        return q

    products = grids.get_pager(query, config)
    g = forms.AlchemyGrid(
        rattail.Product, products, config,
        request.route_url('products'))

    g.configure(
        include=[
            g.upc.with_renderer(UpcFieldRenderer).label("UPC"),
            g.brand,
            g.description,
            ],
        readonly=True)

    grid = g.render(class_='clickable products')
    return grids.render_grid(request, grid, search)


def includeme(config):
    config.add_route('products', '/products')
    config.scan(__name__)
